package com.sd.business.service.inventory.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.in.po.InOutStorageDetails;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.po.InventoryBackup;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.mapper.inventory.InventoryMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageDetailsService;
import com.sd.business.service.inventory.InventoryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Slf4j
@Service
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    private final static BigDecimal tempUnitPrice = BigDecimal.TEN;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private InOutStorageDetailsService inOutStorageDetailsService;

    @Override
    public Page<InventoryVo> getPage(InventorySelectDto dto, String tableName) {
        IWrapper<Inventory> wrapper = getWrapper();

        wrapper.orderByDesc("i", Inventory::getId);
        wrapper.eq("b", Bom::getBomClassifyId, dto.getBomClassifyId());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("b", Bom::getSpecies, dto.getBomSpecies());
        wrapper.eq("b", Bom::getChromatophore, dto.getBomChromatophore());
        wrapper.eq("b", Bom::getEmbossingProcess, dto.getBomEmbossingProcess());
        wrapper.eq("b", Bom::getFrontGrain, dto.getBomFrontGrain());
        wrapper.eq("b", Bom::getReverseGrain, dto.getBomReverseGrain());
        wrapper.like("bs", BomSpec::getColour, dto.getBomSpecColour());
        wrapper.like("bs", BomSpec::getLength, dto.getBomSpecLength());
        wrapper.like("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("i", Inventory::getWarehouseId, dto.getWarehouseId());
        wrapper.eq("i", Inventory::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("i", InventoryBackup::getBackupDate, dto.getBackupDate());

        return this.baseMapper.getPage(dto.getPage(), wrapper, tableName);
    }

    @Override
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto) {
        return baseMapper.getQuantityByWarehouse(dto);
    }

    @Override
    public List<QuantityByDepartmentVo> getQuantityByDepartment() {
        return baseMapper.getQuantityByDepartment();
    }

    @Override
    public void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 保存结存单价批次
            List<InOutStorageDetails> inOutStorageDetailsList = list.stream().map(item -> {
                InOutStorageDetails inOutStorageDetails = new InOutStorageDetails();
                inOutStorageDetails.setDepartmentId(departmentId);
                inOutStorageDetails.setWarehouseId(warehouseId);
                inOutStorageDetails.setBomSpecId(item.getBomSpecId());
                inOutStorageDetails.setInUnitPrice(tempUnitPrice);
                inOutStorageDetails.setQuantity(item.getQuantity());
                return inOutStorageDetails;
            }).collect(Collectors.toList());
            inOutStorageDetailsService.saveBatch(inOutStorageDetailsList);

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);

            // 通过出库id获取bom规格结存列表
            Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);

            // 通过bom规格分组
            Map<Long, Inventory> map = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            for (InOutFun inOutFun : list) {

                // 库存
                Inventory inventory = map.computeIfAbsent(inOutFun.getBomSpecId(), item -> {
                    Inventory tempInventory = new Inventory();
                    tempInventory.setWarehouseId(warehouseId);
                    tempInventory.setDepartmentId(departmentId);
                    tempInventory.setBomSpecId(item);
                    tempInventory.setQuantity(BigDecimal.ZERO);
                    inventoryList.add(tempInventory);
                    return tempInventory;
                });

                // 获取入库结存单价
                BigDecimal balanceUnitPrice = getBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);

                inventory.setQuantity(inventory.getQuantity().add(inOutFun.getQuantity()));
                inventory.setBalanceUnitPrice(balanceUnitPrice);
            }

            saveOrUpdateBatch(inventoryList);
        }
    }

    @Override
    public void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);
            Map<Long, Inventory> inventoryMap = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            // 获取bom规格id和结存明细列表map
            Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);

            for (InOutFun inOutFun : list) {
                Long bomSpecId = inOutFun.getBomSpecId();
                BigDecimal quantity = inOutFun.getQuantity();

                Inventory inventory = inventoryMap.get(bomSpecId);
                if (inventory == null || inventory.getQuantity().compareTo(quantity) < 0) {
                    BomSpec bomSpec = bomSpecService.getById(bomSpecId);
                    if (bomSpec == null) {
                        throw new ServiceException("未知bom规格id：" + bomSpecId);
                    }
                    throw new ServiceException("出库失败,品名为：" + bomSpec.getName() + "的bom库存不足");
                }

                // 计算出库结存单价
                BigDecimal balanceUnitPrice = getOutBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);

                // 赋值结存单价
                inventory.setBalanceUnitPrice(balanceUnitPrice);
                inventory.setQuantity(inventory.getQuantity().subtract(quantity));
            }

            // 修改计算结存单价的出入库明细表
            List<InOutStorageDetails> inOutStorageDetailsList = inOutStorageDetailsMap.values()
                    .stream().flatMap(Collection::stream).collect(Collectors.toList());
            inOutStorageDetailsService.updateBatchById(inOutStorageDetailsList);

            // 修改库存数量以及结存单价
            updateBatchById(inventoryList);
        }
    }

    /**
     * 通过事业部id和出库id获取bom规格库存
     */
    private List<Inventory> getInventoryList(Long departmentId, Long warehouseId, List<Long> bomSpecIdList) {
        return list(q -> q
                .eq(Inventory::getDepartmentId, departmentId)
                .eq(Inventory::getWarehouseId, warehouseId)
                .in(Inventory::getBomSpecId, bomSpecIdList)
        );
    }

    /**
     * 通过出库id获取bom规格结存列表
     */
    private Map<Long, List<InOutStorageDetails>> getInOutStorageDetailsMap(Long warehouseId, List<Long> bomSpecIdList) {
        List<InOutStorageDetails> list = inOutStorageDetailsService.list(q -> q
                .eq(InOutStorageDetails::getWarehouseId, warehouseId)
                .in(InOutStorageDetails::getBomSpecId, bomSpecIdList)
                .gt(InOutStorageDetails::getQuantity, BigDecimal.ZERO)
        );
        return list.stream().collect(Collectors.groupingBy(InOutStorageDetails::getBomSpecId));
    }

    /**
     * 获取入库结存单价
     */
    private BigDecimal getBalanceUnitPrice(Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap, InOutFun inOutFun) {

        // 获取结存数量
        BigDecimal sumQuantity = inOutStorageDetailsMap
                .getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList())
                .stream()
                .map(InOutStorageDetails::getQuantity)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 获取结存金额
        BigDecimal sumPrice = inOutStorageDetailsMap
                .getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList())
                .stream()
                .map(item -> item.getQuantity().multiply(item.getInUnitPrice()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 结存单价
        if (sumQuantity.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        }

        // 计算新结存单价
        return sumPrice.divide(sumQuantity, 2, RoundingMode.HALF_UP);
    }

    /**
     * 获取出库结存单价
     */
    private BigDecimal getOutBalanceUnitPrice(Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap, InOutFun inOutFun) {

        Long bomSpecId = inOutFun.getBomSpecId();
        BigDecimal quantity = inOutFun.getQuantity();

        List<InOutStorageDetails> outStorageDetails = inOutStorageDetailsMap.getOrDefault(bomSpecId, Collections.emptyList());

        // 循环次数
        int j = 0;

        do {

            if (outStorageDetails.size() == j) {
                throw new ServiceException("数据异常，无法计算库存单价");
            }

            // 获取这条记录剩余的数量
            InOutStorageDetails inOutStorageDetails = outStorageDetails.get(j);

            // 单条可出库数量
            BigDecimal tempQuantity = inOutStorageDetails.getQuantity();

            if (tempQuantity.compareTo(quantity) >= 0) {
                inOutStorageDetails.setQuantity(tempQuantity.subtract(quantity));
                quantity = BigDecimal.ZERO;
            } else {
                inOutStorageDetails.setQuantity(BigDecimal.ZERO);
                quantity = quantity.subtract(tempQuantity);
            }

            j++;

        } while (quantity.compareTo(BigDecimal.ZERO) > 0);

        return getBalanceUnitPrice(inOutStorageDetailsMap, inOutFun);
    }

}
