package com.sd.business.service.production.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.StockPreparationDto;
import com.sd.business.entity.production.po.ProductionTask;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.StockPreparationVo;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.production.StockPreparationMapper;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionTaskService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.production.StockPreparationService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StockPreparationServiceImpl implements StockPreparationService {

    @Resource
    private StockPreparationMapper stockPreparationMapper;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private ProductionTaskService productionTaskService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private SkuService skuService;

    @Override
    public List<StockPreparationVo> uncompletedList(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = createWrapper(dto);
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.STOCK_PREPARATION.getKey());
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.NO);
        return stockPreparationMapper.uncompletedList(wrapper);
    }

    @Override
    public Page<StockPreparationVo> completedPage(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = createWrapper(dto);
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.YES);
        wrapper.orderByDesc("os", OrderSku::getStockPreparationTime);
        wrapper.orderByAsc("os", OrderSku::getId);
        return stockPreparationMapper.completedPage(dto.getPage(), wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized void submit(List<Long> orderSkuIdList) {

        // 更爱状态为已备料
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q
                .select(BaseIdPo::getId, OrderSku::getStockPreparationStatus, OrderSku::getOrderId)
                .eq(OrderSku::getStockPreparationStatus, StatusConstant.NO)
                .in(BaseIdPo::getId, orderSkuIdList));

        if (orderSkuIdList.size() == 0) {
            return;
        }

        Date stockPreparationTime = new Date();
        for (OrderSku orderSku : orderSkuList) {
            orderSku.setStockPreparationTime(stockPreparationTime);
            orderSku.setStockPreparationStatus(StatusConstant.YES);
        }
        orderSkuService.updateBatchById(orderSkuList);

        List<Long> orderIdList = orderSkuList.stream().map(OrderSku::getOrderId).distinct().collect(Collectors.toList());
        List<OrderSku> list = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        Map<Long, List<OrderSku>> map = list.stream().collect(Collectors.groupingBy(OrderSku::getOrderId));

        List<OrderInfo> orderInfoList = new ArrayList<>();
        List<ProductionTask> productionTaskList = new ArrayList<>();
        List<ProductionWorkOrder> productionWorkOrderList = new ArrayList<>();

        map.forEach((orderId, itemOrderSkuList) -> {

            // 订单中未备料数量
            long count = itemOrderSkuList.stream()
                    .map(OrderSku::getStockPreparationStatus)
                    .filter(StatusConstant.NO::equals)
                    .count();

            // 存在未备料的订单sku，不执行以下逻辑
            if (count > 0) {
                return;
            }

            // 订单状态改为 生产中
            addOrderInfo(orderId, orderInfoList);

            // 添加 生产任务、工单
            addTaskAndWorkOrder(orderId, itemOrderSkuList, productionTaskList, productionWorkOrderList);

            // 出库


        });

        if (orderInfoList.size() > 0) {
            orderService.updateBatchById(orderInfoList);
        }
        if (productionTaskList.size() > 0) {
            productionTaskService.saveBatch(productionTaskList);
        }
        if (productionWorkOrderList.size() > 0) {
            productionWorkOrderService.saveBatch(productionWorkOrderList);
        }

    }

    private IWrapper<StockPreparationVo> createWrapper(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = IWrapper.getWrapper();

        wrapper.eq("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.eq("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());

        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());

        wrapper.eq("d", Department::getId, dto.getDepartmentId());

        wrapper.eq("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.eq("ss", SkuSpec::getName, dto.getSkuSpecName());

        wrapper.eq("bs", BomSpec::getWidth, dto.getWidth());

        return wrapper;
    }

    /**
     * 订单状态改为 生产中
     */
    private void addOrderInfo(Long orderId, List<OrderInfo> orderInfoList) {
        OrderInfo orderInfo = new OrderInfo();
        orderInfo.setId(orderId);
        orderInfo.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
        orderInfoList.add(orderInfo);
    }

    /**
     * 添加 生产任务、工单
     */
    private void addTaskAndWorkOrder(Long orderId,
                                     List<OrderSku> orderSkuList,
                                     List<ProductionTask> productionTaskList,
                                     List<ProductionWorkOrder> productionWorkOrderList) {

        OrderInfo orderInfo = orderService.getById(orderId);
        Map<String, ProductionTask> skuSpecIdProductionTaskMap = new HashMap<>();
        int num = 1;

        for (OrderSku orderSku : orderSkuList) {

            Long skuSpecId = orderSku.getSkuSpecId();
            Long bomSpecId = orderSku.getBomSpecId();
            Integer printType = orderSku.getPrintType();
            BigDecimal quantity = orderSku.getQuantity();

            ProductionTask productionTask = skuSpecIdProductionTaskMap.computeIfAbsent(
                    skuSpecId + ":" + bomSpecId + ":" + printType,
                    item -> {
                        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
                        Sku sku = skuService.getById(skuSpec.getSkuId());

                        ProductionTask tempProductionTask = new ProductionTask();
                        tempProductionTask.setId(IdWorker.getId());
                        tempProductionTask.setOrderId(orderSku.getOrderId());
                        tempProductionTask.setSkuSpecId(skuSpecId);
                        tempProductionTask.setBomSpecId(bomSpecId);
                        tempProductionTask.setCraftProductionLineId(sku.getCraftProductionLineId());
                        tempProductionTask.setMachinedPanel(skuSpec.getMachinedPanel());
                        tempProductionTask.setPrintType(printType);
                        tempProductionTask.setProductionQuantity(BigDecimal.ZERO);
                        tempProductionTask.setCompleteQuantity(BigDecimal.ZERO);

                        productionTaskList.add(tempProductionTask);

                        return tempProductionTask;
                    });

            productionTask.setProductionQuantity(productionTask.getProductionQuantity().add(quantity));

            for (int i = 0; i < quantity.intValue(); i++) {
                ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
                productionWorkOrder.setProductionTaskId(productionTask.getId());
                productionWorkOrder.setCode(orderInfo.getCode() + String.format("-%05d", num));
                productionWorkOrder.setStatus(0);
                productionWorkOrderList.add(productionWorkOrder);
                num++;
            }

        }
    }

}
