package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.sku.dto.SkuDto;
import com.sd.business.entity.sku.dto.SkuSelectDto;
import com.sd.business.entity.sku.dto.SkuSpecDto;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.sku.vo.SkuSpecLinkVo;
import com.sd.business.entity.sku.vo.SkuSpecVo;
import com.sd.business.entity.sku.vo.SkuVo;
import com.sd.business.mapper.sku.SkuMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * <p>
 * sku 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class SkuServiceImpl extends ServiceImpl<SkuMapper, Sku> implements SkuService {

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<SkuVo> getPage(SkuSelectDto dto) {
        IWrapper<Sku> wrapper = getWrapper();
        wrapper.orderByDesc("s", Sku::getId);
        wrapper.like("s", Sku::getCode, dto.getCode());
        wrapper.like("s", Sku::getName, dto.getName());
        wrapper.eq("s", Sku::getType, dto.getType());
        wrapper.eq("s", Sku::getBrand, dto.getBrand());
        Page<SkuVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<SkuVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        List<Long> skuIdList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<SkuSpec>> skuSpecMap = skuSpecService.mapKGroup(SkuSpec::getSkuId,
                q -> q.in(SkuSpec::getSkuId, skuIdList));
        records.forEach(item -> item.setSkuSpecList(skuSpecMap.getOrDefault(item.getId(), Collections.emptyList())));
        return page;
    }

    @Override
    public SkuVo detail(Long id) {
        // sku
        Sku sku = this.getById(id);
        Assert.notNull(sku, "未通过skuId找到sku");
        SkuVo result = BeanUtil.toBean(sku, SkuVo.class);

        // sku规格
        List<SkuSpec> list = skuSpecService.list(q -> q.in(SkuSpec::getSkuId, id));
        List<SkuSpecVo> skuSpecList = BeanUtil.copyToList(list, SkuSpecVo.class);
        result.setSkuSpecList(skuSpecList);

        // sku规格关联
        List<SkuSpecLink> skuSpecLinkList = skuSpecLinkService.list(
                q -> q.eq(SkuSpecLink::getDepartmentId, SecurityUtils.getDeptId()).eq(SkuSpecLink::getSkuId, id));

        // sku规格关联map
        Map<Long, Map<Integer, List<SkuSpecLink>>> map = skuSpecLinkList.stream().collect(
                Collectors.groupingBy(SkuSpecLink::getSkuSpecId, Collectors.groupingBy(SkuSpecLink::getType)));

        // bom规格id列表
        List<Long> bomSpecIdList = new ArrayList<>();
        bomSpecIdList.addAll(list.stream().map(SkuSpec::getBomSpecId).filter(ObjectUtil::isNotNull).collect(Collectors.toList()));
        bomSpecIdList.addAll(skuSpecLinkList.stream().map(SkuSpecLink::getBomSpecId).collect(Collectors.toList()));
        Map<Long, BomSpec> bomSpecMap = new HashMap<>();
        if (bomSpecIdList.size() > 0) {
            bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIdList));
        }

        for (SkuSpecVo item : skuSpecList) {

            // 赋值主材bom名称
            Long bomSpecId = item.getBomSpecId();
            if (bomSpecId != null) {
                BomSpec bomSpec = bomSpecMap.get(bomSpecId);
                if (bomSpec != null) {
                    item.setBomSpecName(bomSpec.getName());
                }
            }

            // 包材，快递包装
            Map<Integer, List<SkuSpecLink>> typeSkuSpecLinkMap = map.get(item.getId());
            if (typeSkuSpecLinkMap == null) {
                item.setPackagingMaterialList(Collections.emptyList());
                item.setExpressPackingList(Collections.emptyList());
                continue;
            }

            // 赋值包材
            List<SkuSpecLinkVo> packagingMaterialList = createSkuSpecLinkVoList(typeSkuSpecLinkMap.get(1), bomSpecMap);
            item.setPackagingMaterialList(packagingMaterialList);

            // 赋值快递包装
            List<SkuSpecLinkVo> expressPackingList = createSkuSpecLinkVoList(typeSkuSpecLinkMap.get(2), bomSpecMap);
            item.setExpressPackingList(expressPackingList);

        }

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(SkuDto skuDto) {

        skuDto.setSource(1);
        save(skuDto);

        List<SkuSpec> skuSpecList = skuDto.getSkuSpecList().stream()
                .peek(item -> item.setSkuId(skuDto.getId()))
                .map(item -> (SkuSpec) item)
                .collect(Collectors.toList());
        skuSpecService.saveBatch(skuSpecList);

        List<SkuSpecLink> skuSpecLinkList = createSkuSpecLink(skuDto.getSkuSpecList());
        skuSpecLinkService.saveBatch(skuSpecLinkList);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(SkuDto skuDto) {

        Long skuId = skuDto.getId();
        List<SkuSpecDto> skuSpecDtoList = skuDto.getSkuSpecList();

        // 编辑sku
        updateById(skuDto);

        // 编辑sku规格
        editSkuSpecList(skuId, skuSpecDtoList);

        // 编辑sku规格关联快递和包材
        editSkuSpecLinkList(skuId, skuSpecDtoList);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        removeById(id);
        skuSpecService.remove(q -> q.eq(SkuSpec::getSkuId, id));
        skuSpecLinkService.remove(q -> q.eq(SkuSpecLink::getSkuId, id));
    }

    /**
     * 创建sku规格关联
     */
    private List<SkuSpecLinkVo> createSkuSpecLinkVoList(List<SkuSpecLink> skuSpecLinkList, Map<Long, BomSpec> bomSpecMap) {

        if (ObjectUtil.isEmpty(skuSpecLinkList)) {
            return Collections.emptyList();
        }

        return skuSpecLinkList.stream().map(item -> {
            SkuSpecLinkVo skuSpecLinkVo = BeanUtil.toBean(item, SkuSpecLinkVo.class);
            BomSpec bomSpec = bomSpecMap.get(item.getBomSpecId());
            if (bomSpec != null) {
                skuSpecLinkVo.setName(bomSpec.getName());
            }
            return skuSpecLinkVo;
        }).collect(Collectors.toList());

    }

    /**
     * 编辑sku规格
     */
    private void editSkuSpecList(Long skuId, List<SkuSpecDto> skuSpecDtoList) {

        List<Long> skuSpecIdList = skuSpecDtoList.stream().map(SkuSpec::getId)
                .filter(ObjectUtil::isNotNull).collect(Collectors.toList());

        skuSpecService.remove(q -> q
                .eq(SkuSpec::getSkuId, skuId)
                .notIn(ObjectUtil.isNotEmpty(skuSpecIdList), BaseIdPo::getId, skuSpecIdList));

        List<SkuSpec> skuSpecList = skuSpecDtoList.stream()
                .peek(item -> item.setSkuId(skuId))
                .map(item -> (SkuSpec) item)
                .collect(Collectors.toList());
        skuSpecService.saveOrUpdateBatch(skuSpecList);

    }

    /**
     * 编辑sku规格关联快递和包材
     */
    private void editSkuSpecLinkList(Long skuId, List<SkuSpecDto> skuSpecDtoList) {

        List<Long> skuSpecLinkIdList = skuSpecDtoList.stream()
                .flatMap(item -> Stream.concat(
                        item.getPackagingMaterialList().stream(),
                        item.getExpressPackingList().stream()))
                .map(BaseIdPo::getId)
                .filter(ObjectUtil::isNotNull)
                .collect(Collectors.toList());

        skuSpecLinkService.remove(q -> q
                .eq(SkuSpecLink::getSkuId, skuId)
                .eq(SkuSpecLink::getDepartmentId, SecurityUtils.getDeptId())
                .notIn(ObjectUtil.isNotEmpty(skuSpecLinkIdList), BaseIdPo::getId, skuSpecLinkIdList));

        List<SkuSpecLink> skuSpecLinkList = createSkuSpecLink(skuSpecDtoList);
        skuSpecLinkService.saveOrUpdateBatch(skuSpecLinkList);

    }

    /**
     * 创建sku规格关联快递和包材
     */
    private List<SkuSpecLink> createSkuSpecLink(List<SkuSpecDto> skuSpecDtoList) {

        return skuSpecDtoList.stream().flatMap(item -> {

            List<SkuSpecLink> packagingMaterialList = item.getPackagingMaterialList();
            for (SkuSpecLink skuSpecLink : packagingMaterialList) {
                skuSpecLink.setSkuId(item.getSkuId());
                skuSpecLink.setSkuSpecId(item.getId());
                skuSpecLink.setDepartmentId(SecurityUtils.getDeptId());
                skuSpecLink.setType(1);
            }

            List<SkuSpecLink> expressPackingList = item.getExpressPackingList();
            for (SkuSpecLink skuSpecLink : expressPackingList) {
                skuSpecLink.setSkuId(item.getSkuId());
                skuSpecLink.setSkuSpecId(item.getId());
                skuSpecLink.setDepartmentId(SecurityUtils.getDeptId());
                skuSpecLink.setType(2);
            }

            return Stream.concat(packagingMaterialList.stream(), expressPackingList.stream());

        }).collect(Collectors.toList());

    }

}
